@@ -4,18 +4,118 @@
 from typing import Any, Callable, Dict, Optional
 
 import vapoursynth as vs
-from vsutil import depth, fallback, get_w, get_y, join, plane
+from vsutil import fallback, get_w, get_y, join, plane
 
 from . import kernels, util
+from .misc import scale_thresh
 
 core = vs.core
 
 
+def clamp_aa(src: vs.VideoNode, weak: vs.VideoNode, strong: vs.VideoNode, strength: float = 1) -> vs.VideoNode:
+    """
+    Clamp stronger AAs to weaker AAs.
+
+    Stolen from Zastin.
+
+    :param src:      Non-AA'd source clip.
+    :param weak:     Weakly-AA'd clip (eg: nnedi3)
+    :param strong:   Strongly-AA'd clip (eg: eedi3)
+    :param strength: Clamping strength (Default: 1)
+
+    :return:         Clip with clamped anti-aliasing.
+    """
+    if src.format is None or weak.format is None or strong.format is None:
+        raise ValueError("nneedi3_clamp: 'Variable-format clips not supported'")
+    thr = strength * (1 << (src.format.bits_per_sample - 8)) if src.format.sample_type == vs.INTEGER \
+        else strength/219
+    clamp = core.std.Expr([get_y(src), get_y(weak), get_y(strong)],
+                          expr=f"x y - x z - xor x x y - abs x z - abs < z y {thr} + min y {thr} - max z ? ?"
+                          if thr != 0 else "x y z min max y z max min")
+    return clamp if src.format.color_family == vs.GRAY \
+        else core.std.ShufflePlanes([clamp, src], planes=[0, 1, 2], colorfamily=vs.YUV)
+
+
+def taa(clip: vs.VideoNode, aafun: Callable[[vs.VideoNode], vs.VideoNode]) -> vs.VideoNode:
+    """
+    Perform transpose AA.
+
+    :param clip:   Input clip.
+    :param aafun:  Antialiasing function
+
+    :return:       Antialiased clip
+    """
+    if clip.format is None:
+        raise ValueError("taa: 'Variable-format clips not supported'")
+
+    y = get_y(clip)
+
+    aa = aafun(y.std.Transpose())
+    aa = aa.resize.Spline36(height=clip.width, src_top=0.5).std.Transpose()
+    aa = aafun(aa)
+    aa = aa.resize.Spline36(height=clip.height, src_top=0.5)
+
+    return aa if clip.format.color_family == vs.GRAY \
+        else core.std.ShufflePlanes([aa, clip], planes=[0, 1, 2], colorfamily=vs.YUV)
+
+
+def nnedi3(clip: vs.VideoNode, opencl: bool = False, **override: Any) -> vs.VideoNode:
+    """
+    Standard nnedi3 antialiasing.
+
+    :param clip:     Input clip
+    :param opencl:   Use OpenCL (Default: False)
+    :param override: nnedi3 parameter overrides
+
+    :return:         Antialiased clip
+    """
+    nnedi3_args: Dict[str, Any] = dict(field=0, dh=True, nsize=3, nns=3, qual=1)
+    nnedi3_args.update(override)
+
+    def _nnedi3(clip: vs.VideoNode) -> vs.VideoNode:
+        return clip.nnedi3cl.NNEDI3CL(**nnedi3_args) if opencl \
+            else clip.nnedi3.nnedi3(**nnedi3_args)
+
+    return taa(clip, _nnedi3)
+
+
+def eedi3(clip: vs.VideoNode, opencl: bool = False, **override: Any) -> vs.VideoNode:
+    """
+    Standard eedi3 antialiasing.
+
+    :param clip:     Input clip
+    :param opencl:   Use OpenCL (Default: False)
+    :param override: eedi3 parameter overrides
+
+    :return:         Antialiased clip
+    """
+    eedi3_args: Dict[str, Any] = dict(field=0, dh=True, alpha=0.25, beta=0.5, gamma=40, nrad=2, mdis=20)
+    eedi3_args.update(override)
+
+    def _eedi3(clip: vs.VideoNode) -> vs.VideoNode:
+        return clip.eedi3m.EEDI3CL(**eedi3_args) if opencl \
+            else clip.eedi3m.EEDI3(**eedi3_args)
+
+    return taa(clip, _eedi3)
+
+
+def kirsch_aa_mask(clip: vs.VideoNode, mthr: float = 0.25) -> vs.VideoNode:
+    """
+    Kirsh-based AA mask.
+
+    :param clip: Input clip
+    :param mthr: Mask threshold [0, 1] (Default: 0.25)
+    """
+    try:
+        from kagefunc import kirsch
+    except ModuleNotFoundError:
+        raise ModuleNotFoundError("nnedi3_clamp: missing dependency 'kagefunc'")
+    return kirsch(get_y(clip)).std.Binarize(scale_thresh(mthr, clip)).std.Maximum().std.Convolution([1] * 9)
+
+
 def nneedi3_clamp(clip: vs.VideoNode, strength: float = 1,
-                  mask: Optional[vs.VideoNode] = None, ret_mask: bool = False,
-                  mthr: float = 0.25, show_mask: bool = False,
-                  opencl: bool = False, nnedi3cl: Optional[bool] = None,
-                  eedi3cl: Optional[bool] = None) -> vs.VideoNode:
+                  mask: Optional[vs.VideoNode] = None,
+                  mthr: float = 0.25, opencl: bool = False) -> vs.VideoNode:
     """
     A function that clamps eedi3 to nnedi3 for the purpose of reducing eedi3 artifacts.
     This should fix every issue created by eedi3. For example: https://i.imgur.com/hYVhetS.jpg
@@ -23,8 +123,7 @@ def nneedi3_clamp(clip: vs.VideoNode, strength: float = 1,
     Original function written by Zastin, modified by LightArrowsEXE.
 
     Dependencies:
-    * kagefunc (optional: retinex edgemask)
-    * vapoursynth-retinex (optional: retinex edgemask)
+    * kagefunc (optional: automatic masking, otherwise mask must be user-supplied)
     * vapoursynth-tcanny (optional: retinex edgemask)
     * vapoursynth-eedi3
     * vapoursynth-nnedi3
@@ -34,84 +133,16 @@ def nneedi3_clamp(clip: vs.VideoNode, strength: float = 1,
     :param strength:            Set threshold strength for over/underflow value for clamping eedi3's result
                                 to nnedi3 +/- strength * 256 scaled to 8 bit (Default: 1)
     :param mask:                Clip to use for custom mask (Default: None)
-    :param ret_mask:            Replace default mask with a retinex edgemask (Default: False)
     :param mthr:                Binarize threshold for the mask, scaled to float (Default: 0.25)
-    :param show_mask:           Return mask instead of clip (Default: False)
     :param opencl:              OpenCL acceleration (Default: False)
-    :param nnedi3cl:            OpenCL acceleration for nnedi3 (Default: False)
-    :param eedi3cl:             OpenCL acceleration for eedi3 (Default: False)
 
     :return:                    Antialiased clip
     """
-
     if clip.format is None:
         raise ValueError("nneedi3_clamp: 'Variable-format clips not supported'")
-
-    clip_y = get_y(clip)
-
-    bits = clip.format.bits_per_sample
-    sample_type = clip.format.sample_type
-    shift = bits - 8
-    thr = strength * (1 >> shift) if sample_type == vs.INTEGER else strength/219
-    expr = 'x z - y z - xor y x y {0} + min y {0} - max ?'.format(thr)
-
-    if sample_type == vs.INTEGER:
-        mthr = round(mthr * ((1 >> shift) - 1))
-
-    if mask is None:
-        try:
-            from kagefunc import kirsch
-        except ModuleNotFoundError:
-            raise ModuleNotFoundError("nnedi3_clamp: missing dependency 'kagefunc'")
-        mask = kirsch(clip_y)
-        if ret_mask:
-            # workaround to support float input
-            ret = depth(clip_y, min(16, bits))
-            ret = core.retinex.MSRCP(ret, sigma=[50, 200, 350], upper_thr=0.005)
-            ret = depth(ret, bits)
-            tcanny = core.tcanny.TCanny(ret, mode=1, sigma=[1.0])
-            tcanny = core.std.Minimum(tcanny, coordinates=[1, 0, 1, 0, 0, 1, 0, 1])
-            # no clamping needed when binarizing
-            mask = core.std.Expr([mask, tcanny], 'x y +')
-        mask = mask.std.Binarize(thr).std.Maximum().std.Convolution([1] * 9)
-
-    nnedi3cl = fallback(nnedi3cl, opencl)
-    eedi3cl = fallback(eedi3cl, opencl)
-
-    nnedi3_args: Dict[str, Any] = dict(nsize=3, nns=3, qual=1)
-    eedi3_args: Dict[str, Any] = dict(alpha=0.25, beta=0.5, gamma=40, nrad=2, mdis=20)
-
-    clip_tra = core.std.Transpose(clip_y)
-
-    if eedi3cl:
-        strong = core.eedi3m.EEDI3CL(clip_tra, 0, True, **eedi3_args)
-        strong = core.resize.Spline36(strong, height=clip.width, src_top=0.5)
-        strong = core.std.Transpose(strong)
-        strong = core.eedi3m.EEDI3CL(strong, 0, True, **eedi3_args)
-        strong = core.resize.Spline36(strong, height=clip.height, src_top=0.5)
-    else:
-        strong = core.eedi3m.EEDI3(clip_tra, 0, True, mclip=mask.std.Transpose(), **eedi3_args)
-        strong = core.resize.Spline36(strong, height=clip.width, src_top=0.5)
-        strong = core.std.Transpose(strong)
-        strong = core.eedi3m.EEDI3(strong, 0, True, mclip=mask, **eedi3_args)
-        strong = core.resize.Spline36(strong, height=clip.height, src_top=0.5)
-
-    nnedi3: Callable[..., vs.VideoNode] = core.nnedi3.nnedi3
-    if nnedi3cl:
-        nnedi3 = core.nnedi3cl.NNEDI3CL
-
-    weak = nnedi3(clip_tra, 0, True, **nnedi3_args)
-    weak = core.resize.Spline36(weak, height=clip.width, src_top=0.5)
-    weak = core.std.Transpose(weak)
-    weak = nnedi3(weak, 0, True, **nnedi3_args)
-    weak = core.resize.Spline36(weak, height=clip.height, src_top=0.5)
-
-    aa = core.std.Expr([strong, weak, clip_y], expr)
-
-    merged = core.std.MaskedMerge(clip_y, aa, mask)
-
-    if show_mask:
-        return mask
+    y = get_y(clip)
+    merged = core.std.MaskedMerge(y, clamp_aa(y, nnedi3(y, opencl=opencl), eedi3(y, opencl=opencl)),
+                                  mask or kirsch_aa_mask(y))
     return merged if clip.format.color_family == vs.GRAY else core.std.ShufflePlanes([merged, clip], [0, 1, 2], vs.YUV)
 
 
@@ -138,32 +169,21 @@ def transpose_aa(clip: vs.VideoNode,
 
     clip_y = get_y(clip)
 
-    if eedi3:
-        def _aa(clip_y: vs.VideoNode) -> vs.VideoNode:
-            clip_y = clip_y.std.Transpose()
-            clip_y = clip_y.eedi3m.EEDI3(0, 1, 0, 0.5, 0.2)
-            clip_y = clip_y.znedi3.nnedi3(1, 0, 0, 3, 4, 2)
-            clip_y = clip_y.resize.Bicubic(clip.height, clip.width, src_top=.5)
-            clip_y = clip_y.std.Transpose()
-            clip_y = clip_y.eedi3m.EEDI3(0, 1, 0, 0.5, 0.2)
-            clip_y = clip_y.znedi3.nnedi3(1, 0, 0, 3, 4, 2)
-            return clip_y.resize.Bicubic(clip.width, clip.height, src_top=.5)
-    else:
-        def _aa(clip_y: vs.VideoNode) -> vs.VideoNode:
-            clip_y = clip_y.std.Transpose()
-            clip_y = clip_y.nnedi3.nnedi3(0, 1, 0, 3, 3, 2)
-            clip_y = clip_y.nnedi3.nnedi3(1, 0, 0, 3, 3, 2)
-            clip_y = clip_y.resize.Bicubic(clip.height, clip.width, src_top=.5)
-            clip_y = clip_y.std.Transpose()
-            clip_y = clip_y.nnedi3.nnedi3(0, 1, 0, 3, 3, 2)
-            clip_y = clip_y.nnedi3.nnedi3(1, 0, 0, 3, 3, 2)
-            return clip_y.resize.Bicubic(clip.width, clip.height, src_top=.5)
+    def _aafun(clip: vs.VideoNode) -> vs.VideoNode:
+        return clip.eedi3m.EEDI3(0, 1, 0, 0.5, 0.2).znedi3.nnedi3(1, 0, 0, 3, 4, 2) if eedi3 \
+            else clip.nnedi3.nnedi3(0, 1, 0, 3, 3, 2).nnedi3.nnedi3(1, 0, 0, 3, 3, 2)
+
+    def _taa(clip: vs.VideoNode) -> vs.VideoNode:
+        aa = _aafun(clip.std.Transpose())
+        aa = aa.resize.Bicubic(clip.height, clip.width, src_top=.5).std.Transpose()
+        aa = _aafun(aa)
+        return aa.resize.Bicubic(clip.width, clip.height, src_top=.5)
 
     def _csharp(flt: vs.VideoNode, clip: vs.VideoNode) -> vs.VideoNode:
         blur = core.std.Convolution(flt, [1] * 9)
         return core.std.Expr([flt, clip, blur], 'x y < x x + z - x max y min x x + z - x min y max ?')
 
-    aaclip = _aa(clip_y)
+    aaclip = _taa(clip_y)
     aaclip = _csharp(aaclip, clip_y)
     aaclip = util.pick_repair(clip_y)(aaclip, clip_y, rep)
 
@@ -219,62 +239,48 @@ def upscaled_sraa(clip: vs.VideoNode,
 
     luma = get_y(clip)
 
-    nnargs: Dict[str, Any] = dict(nsize=0, nns=4, qual=2)
+    nnargs: Dict[str, Any] = dict(field=0, nsize=0, nns=4, qual=2)
     # TAAmbk defaults are 0.5, 0.2, 20, 3, 30
-    eeargs: Dict[str, Any] = dict(alpha=0.2, beta=0.6, gamma=40, nrad=2, mdis=20)
+    eeargs: Dict[str, Any] = dict(field=0, dh=False, alpha=0.2, beta=0.6, gamma=40, nrad=2, mdis=20)
     eeargs.update(eedi3_args)
 
     if rfactor < 1:
         raise ValueError("upscaled_sraa: '\"rfactor\" must be above 1'")
 
     ssw = round(clip.width * rfactor)
+    ssw = (ssw + 1) & ~1
     ssh = round(clip.height * rfactor)
+    ssh = (ssh + 1) & ~1
 
-    while ssw % 2:
-        ssw += 1
-    while ssh % 2:
-        ssh += 1
+    height = height or clip.height
 
-    if height is None:
-        height = clip.height
     if width is None:
-        if height != clip.height:
-            width = get_w(height, aspect_ratio=clip.width / clip.height)
-        else:
-            width = clip.width
+        width = clip.width if height == clip.height else get_w(height, aspect_ratio=clip.width / clip.height)
 
     nnedi3cl = fallback(nnedi3cl, opencl)
     eedi3cl = fallback(eedi3cl, opencl)
 
-    # there doesn't seem to be a cleaner way to do this that makes mypy happy
-    def nnedi3(*args: Any, **kwargs: Any) -> vs.VideoNode:
-        return core.nnedi3cl.NNEDI3CL(*args, **kwargs) if nnedi3cl else core.nnedi3.nnedi3(*args, **kwargs)
+    def _nnedi3(clip: vs.VideoNode, dh: bool = False) -> vs.VideoNode:
+        return clip.nnedi3cl.NNEDI3CL(dh=dh, **nnargs) if nnedi3cl \
+            else clip.nnedi3.nnedi3(dh=dh, **nnargs)
 
-    def eedi3(*args: Any, **kwargs: Any) -> vs.VideoNode:
-        return core.eedi3m.EEDI3CL(*args, **kwargs) if nnedi3cl else core.eedi3m.EEDI3(*args, **kwargs)
+    def _eedi3(clip: vs.VideoNode, sclip: vs.VideoNode) -> vs.VideoNode:
+        return clip.eedi3m.EEDI3CL(sclip=sclip, **eeargs) if eedi3cl \
+            else clip.eedi3m.EEDI3(sclip=sclip, **eeargs)
 
     # Nnedi3 upscale from source height to source height * rounding (Default 1.5)
-    up_y = nnedi3(luma, 0, 1, 0, **nnargs)
-    up_y = core.resize.Spline36(up_y, height=ssh, src_top=.5)
-    up_y = core.std.Transpose(up_y)
-    up_y = nnedi3(up_y, 0, 1, 0, **nnargs)
-    up_y = core.resize.Spline36(up_y, height=ssw, src_top=.5)
+    up_y = _nnedi3(luma, dh=True)
+    up_y = up_y.resize.Spline36(height=ssh, src_top=0.5).std.Transpose()
+    up_y = _nnedi3(up_y, dh=True)
+    up_y = up_y.resize.Spline36(height=ssw, src_top=0.5)
 
     # Single-rate AA
-    aa_y = eedi3(up_y, 0, 0, 0, sclip=nnedi3(up_y, 0, 0, 0, **nnargs), **eeargs)
+    aa_y = _eedi3(up_y, sclip=_nnedi3(up_y))
     aa_y = core.std.Transpose(aa_y)
-    aa_y = eedi3(aa_y, 0, 0, 0, sclip=nnedi3(aa_y, 0, 0, 0, **nnargs), **eeargs)
+    aa_y = _eedi3(aa_y, sclip=_nnedi3(aa_y))
 
-    scaled: vs.VideoNode
-
-    # Back to source clip height or given height
-    if downscaler is None:
-        scaled = aa_y
-    else:
-        scaled = downscaler(aa_y, width, height)
-
-    if rep:
-        scaled = util.pick_repair(scaled)(scaled, luma.resize.Bicubic(width, height), mode=rep)
+    scaled = aa_y if downscaler is None else downscaler(aa_y, width, height)
+    scaled = util.pick_repair(scaled)(scaled, luma.resize.Bicubic(width, height), mode=rep) if rep else scaled
 
     if clip.format.num_planes == 1 or downscaler is None:
         return scaled
@@ -284,6 +290,6 @@ def upscaled_sraa(clip: vs.VideoNode,
         if width % 2:
             raise ValueError("upscaled_sraa: '\"width\" must be an even number when not passing a GRAY clip'")
 
-        chr = kernels.Bicubic().scale(clip, width, height)
-        return join([scaled, plane(chr, 1), plane(chr, 2)])
+        chroma = kernels.Bicubic().scale(clip, width, height)
+        return join([scaled, plane(chroma, 1), plane(chroma, 2)])
     return join([scaled, plane(clip, 1), plane(clip, 2)])
