@@ -12,17 +12,37 @@ from . import util
 core = vs.core
 
 
+def sivtc(clip: vs.VideoNode, TFF: bool, pattern: int = 0) -> vs.VideoNode:
+    """
+    A very simple fieldmatching function.
+
+    This is essentially a stripped-down JIVTC offering JUST the basic fieldmatching part.
+    As such, you may need to combine multiple instances if patterns change throughout the clip.
+
+    :param clip:        Input clip
+    :param TFF:         Top-Field-First
+    :param pattern:     First frame of any clean-combed-combed-clean-clean sequence
+
+    :return:            IVTC'd clip
+    """
+    pattern = pattern % 5
+
+    defivtc = core.std.SeparateFields(clip, tff=TFF).std.DoubleWeave()
+    selectlist = [[0, 3, 6, 8], [0, 2, 5, 8], [0, 2, 4, 7], [2, 4, 6, 9], [1, 4, 6, 8]]
+    return core.std.SelectEvery(defivtc, 10, selectlist[pattern])
+
+
 def deblend(clip: vs.VideoNode, rep: Optional[int] = None) -> vs.VideoNode:
     """
     A simple function to fix deblending for interlaced video with an AABBA blending pattern,
-    where A is a normal frame and B is a blended frame.
+    where A is a regular frame and B is a blended frame.
 
     Assuming there's a constant pattern of frames (labeled A, B, C, CD, and DA in this function),
-    blending can be fixed by calculating the C frame by getting halves of CD and DA, and using that
-    to fix up CD. DA can then be dropped due to it being an interlaced frame.
+    blending can be fixed by calculating the D frame by getting halves of CD and DA, and using that
+    to fix up CD. DA is then dropped because it's a duplicated frame.
 
-    However, doing this will result in some of the artifacting being added to the deblended frame.
-    We can mitigate this by repairing the frame with the non-blended frame before it.
+    Doing this will result in some of the artifacting being added to the deblended frame,
+    but we can mitigate that by repairing the frame with the non-blended frame before it.
 
     For more information, please refer to this blogpost by torchlight:
     https://mechaweaponsvidya.wordpress.com/2012/09/13/adventures-in-deblending/
@@ -49,8 +69,7 @@ def deblend(clip: vs.VideoNode, rep: Optional[int] = None) -> vs.VideoNode:
                 c, cd, da, a = clip[n - 1], clip[n], clip[n + 1], clip[n + 2]
                 debl = core.std.Expr([c, cd, da, a], expr_cd)
                 return util.pick_repair(clip)(debl, c, rep) if rep else debl
-            else:
-                return clip
+            return clip
 
     debl = core.std.FrameEval(clip, partial(deblend, clip=clip, rep=rep))
     return core.std.DeleteFrames(debl, blends_b).std.AssumeFPS(fpsnum=24000, fpsden=1001)
