@@ -11,6 +11,7 @@ import vapoursynth as vs
 from vsutil import get_depth, get_w, get_y, is_image, scale_value
 
 from .util import get_prop
+from .types import Range
 
 core = vs.core
 
@@ -102,7 +103,7 @@ def source(file: str, ref: Optional[vs.VideoNode] = None,
 
 def replace_ranges(clip_a: vs.VideoNode,
                    clip_b: vs.VideoNode,
-                   ranges: List[Union[int, Tuple[int, int]]]) -> vs.VideoNode:
+                   ranges: Union[Range, List[Range]]) -> vs.VideoNode:
     """
     A replacement for ReplaceFramesSimple that uses ints and tuples rather than a string.
     Frame ranges are inclusive.
@@ -119,7 +120,11 @@ def replace_ranges(clip_a: vs.VideoNode,
 
     :return:           Clip with ranges from clip_a replaced with clip_b
     """
+    if not isinstance(ranges, list):
+        ranges = [ranges]
+
     out = clip_a
+
     for r in ranges:
         if type(r) is tuple:
             start, end = cast(Tuple[int, int], r)
@@ -505,6 +510,22 @@ def colored_clips(amount: int,
     return [core.std.BlankClip(color=color, **blank_clip_args) for color in rgb_color_list]
 
 
+def scale_thresh(thresh: float, clip: vs.VideoNode) -> float:
+    """
+    Scale binarization thresholds from float to int.
+
+    :param thresh: Threshold [0, 1]
+    :param clip:   Clip to scale to
+
+    :return:       Threshold scaled to [0, 2^clip.depth - 1]
+    """
+    if clip.format is None:
+        raise ValueError("scale_thresh: 'Variable-format clips not supported.'")
+    if thresh < 0 or thresh > 1:
+        raise ValueError("scale_thresh: 'Thresholds must be between 0 and 1 (inclusive).'")
+    return thresh if clip.format.sample_type == vs.FLOAT else round(thresh * ((1 << clip.format.bits_per_sample) - 1))
+
+
 # TODO: Write function that only masks px of a certain color/threshold of colors.
 #       Think the magic wand tool in various image-editing programs.
 
